 // **********************************************************************************************
// **** PROGRAM = HOME ALARM SYSTEM (C) 2010 LUCIDSCIENCE.COM
// **** TARGET = ATMEGA88P & 12 DIGIT KEYPAD
// **** CLOCK SPEED = 8 MHZ INTERNAL
// **********************************************************************************************
#include <mega8.h>
#include <delay.h>
#include <sleep.h>

 

// **********************************************************************************************
// **** SEND SHORT BEEP TO PIEZO BUZZER
// **********************************************************************************************
void BEEP1() {
unsigned char ctr1;

// SINGLE SHORT BEEP
for (ctr1 = 0; ctr1 < 50; ctr1++) {
PORTD.0 = 0;
PORTD.1 = 1;
delay_us(400);
PORTD.0 = 1;
PORTD.1 = 0;
delay_us(400);
}
}

 

// **********************************************************************************************
// **** COMMAND CONFIRM BEEP
// **********************************************************************************************
void BEEP2() {
unsigned char ctr1;

// LEDS ON
PORTB.1 = 1;
PORTB.2 = 1;

// BEEP FREQUENCY 1
delay_ms(250);
for (ctr1 = 0; ctr1 < 100; ctr1++) {
PORTD.0 = 0;
PORTD.1 = 1;
delay_us(500);
PORTD.0 = 1;
PORTD.1 = 0;
delay_us(500);
}
delay_ms(100);

// BEEP FREQUENCY 2
for (ctr1 = 0; ctr1 < 200; ctr1++) {
PORTD.0 = 0;
PORTD.1 = 1;
delay_us(200);
PORTD.0 = 1;
PORTD.1 = 0;
delay_us(200);
}
delay_ms(100);

// BEEP FREQUENCY 3
for (ctr1 = 0; ctr1 < 150; ctr1++) {
PORTD.0 = 0;
PORTD.1 = 1;
delay_us(300);
PORTD.0 = 1;
PORTD.1 = 0;
delay_us(300);
}
delay_ms(250);

// LEDS OFF
PORTB.1 = 0;
PORTB.2 = 0;
}

 

// **********************************************************************************************
// **** SEND AVR INTO POWERDOWN MODE
// **********************************************************************************************
void SLEEP() {

// TURN OF LEDS AND BEEP
PORTB.1 = 0;
PORTB.2 = 0;
BEEP2();

// ENTER POWERDOWN MODE
#asm("sei")
sleep_enable();
powerdown();
}

 

// **********************************************************************************************
// **** WAKE FROM SLEEP ON ALARM PIN CHANGE
// **********************************************************************************************
//interrupt [PCINT2] void pin_change_isr2(void) {
//sleep_disable();
//#asm("cli")
//}

interrupt [EXT_INT1] void ext_int1_isr(void)
{
  sleep_disable();
  #asm("cli")
}
 
// Timer1 output compare A interrupt service routine
interrupt [TIM1_COMPA] void timer1_compa_isr(void)
{
// Place your code here

}
// **********************************************************************************************
// **** READ KEYPAD AND RETURN ASCII VALUE
// **********************************************************************************************
unsigned char KEYPAD() {
unsigned char key;
unsigned char scn;
key = 0;

// SCAN KEYS IN COLUMN 1
PORTD.4 = 0;
delay_ms(1);
scn = (PINC & 15);
PORTD.4 = 1;
// CONVERT KEY TO ASCII
if (scn == 14) key = '1';
if (scn == 13) key = '4';
if (scn == 11) key = '7';
if (scn == 7) key = '*';

// SCAN KEYS IN COLUMN 2
PORTD.5 = 0;
delay_ms(1);
scn = (PINC & 15);
PORTD.5 = 1;
// CONVERT KEY TO ASCII
if (scn == 14) key = '2';
if (scn == 13) key = '5';
if (scn == 11) key = '8';
if (scn == 7) key = '0';

// SCAN KEYS IN COLUMN 3
PORTD.6 = 0;
delay_ms(1);
scn = (PINC & 15);
PORTD.6 = 1;
// CONVERT KEY TO ASCII
if (scn == 14) key = '3';
if (scn == 13) key = '6';
if (scn == 11) key = '9';
if (scn == 7) key = '#';

// SEND KEY BEEP
if (key != 0){
for (scn = 0; scn < 50; scn++) {
PORTD.0 = 0;
PORTD.1 = 1;
delay_us(200);
PORTD.0 = 1;
PORTD.1 = 0;
delay_us(200);
}
}

// HOLD WHILE KEYDOWN
scn = 0;
while (scn != 45) {
scn = 0;
PORTD.4 = 0;
delay_ms(1);
scn = scn + (PINC & 15);
PORTD.4 = 1;
PORTD.5 = 0;
delay_ms(1);
scn = scn + (PINC & 15);
PORTD.5 = 1;
PORTD.6 = 0;
delay_ms(1);
scn = scn + (PINC & 15);
PORTD.6 = 1;
}

// RETURN KEY VALUE
delay_ms(100);
return(key);
}

 

// **********************************************************************************************
// **** PROGRAM VARIABLES AND PORT SETUP
// **********************************************************************************************
void main(void) {

// MAIN PROGRAM VARIABLES
unsigned char ctr1;
unsigned char ctr2;
unsigned int ctr3;
unsigned char mode;
unsigned char code[4];

// 3X4 KEYPAD ROW INPUTS
DDRC.0 = 0;
DDRC.1 = 0;
DDRC.2 = 0;
DDRC.3 = 0;
PORTC.0 = 1;
PORTC.1 = 1;
PORTC.2 = 1;
PORTC.3 = 1;

// 3X4 KEYPAD COL OUTPUTS
DDRD.4 = 1;
DDRD.5 = 1;
DDRD.6 = 1;
PORTD.4 = 1;
PORTD.5 = 1;
PORTD.6 = 1;

// GREEN LED OUTPUT
DDRB.1 = 1;

// RED LED OUTPUT
DDRB.2 = 1;

// PIEZO BUZZER OUTPUTS
DDRD.1 = 0;
DDRD.1 = 1;

// ALARM TRIGGER INPUT
DDRD.3 = 0;
PORTD.3 = 1;

// ALARM SIREN OUTPUT
DDRB.0 = 1;

 

// **********************************************************************************************
// **** PROGRAM INITIALIZATION
// **********************************************************************************************

// ENABLE PIN CHANGE INTERRUPT
//EICRA = 0x00;
//EIMSK = 0x00;
//PCICR = 0x04;
//PCMSK2 = 0x80;
//PCIFR = 0x04;
GICR|=0x80;
MCUCR=0x00;
GIFR=0x80;

// SEND STARTUP BEEP
delay_ms(1000);
for (ctr1 = 0; ctr1 < 4; ctr1++) {
BEEP1();
PORTB.1 = 1;
PORTB.2 = 1;
delay_ms(100);
PORTB.1 = 0;
PORTB.2 = 0;
delay_ms(100);
}

// SET SYSTEM TO IDLE MODE
mode = 0;
ctr1 = 0;
ctr2 = 0;
ctr3 = 0;

// SET ACCESS KEYCODE
code[0] = '1';
code[1] = '2';
code[2] = '3';
code[3] = '4';

 

// **********************************************************************************************
// **** MAIN PROGRAM LOOP
// **********************************************************************************************
while (1) {

 

// **********************************************************************************************
// **** MODE 0 : IDLE AND UNARMED
// **********************************************************************************************
while (mode == 0) {

// TURN OFF EXTERNAL ALARM
PORTB.0 = 0;

// TURN ON GREEN LED
PORTB.1 = 1;
PORTB.2 = 0;

// SEARCH FOR KEYCODE
if (ctr1 == 0 && KEYPAD() == code[0]) ctr1 = 1;
if (ctr1 == 1 && KEYPAD() == code[1]) ctr1 = 2;
if (ctr1 == 2 && KEYPAD() == code[2]) ctr1 = 3;
if (ctr1 == 3 && KEYPAD() == code[3]) ctr1 = 4;

// ARM SYSTEM ON KEYCODE
if (ctr1 == 4) {
ctr1 = 0;
ctr2 = 0;
ctr3 = 0;
mode = 1;
}
}

 

// **********************************************************************************************
// **** MODE 1 : ALARM ARMED WITH 60 SECOND COUNTDOWN
// **********************************************************************************************
while (mode == 1) {

// BEEPS FOR 60 SECONDS
for (ctr1 = 0; ctr1 < 60; ctr1++) {
BEEP1();
PORTB.2 = 1;
delay_ms(500);
PORTB.2 = 0;
delay_ms(500);
}

// ARM SYSTEM AND POWER DOWN
SLEEP();
ctr1 = 0;
ctr2 = 0;
ctr3 = 0;
mode = 2;
}

 

// **********************************************************************************************
// **** MODE 2 : ALARM TRIGGERED WITH 60 SECOND COUNTDOWN
// **********************************************************************************************
while (mode == 2) {

// ALARM WARNING BLIP
for (ctr1 = 0; ctr1 < 50; ctr1++) {
PORTD.0 = 0;
PORTD.1 = 1;
PORTB.2 = 1;
delay_us(150);
PORTD.0 = 1;
PORTD.1 = 0;
PORTB.2 = 0;
delay_us(150);
}

// ACTIVATE SIREN WITHIN 60 SECONDS
ctr3++;
if (ctr3 == 400) mode = 3;

// SEARCH FOR KEYCODE
if (ctr2 == 0 && KEYPAD() == code[0]) ctr2 = 1;
if (ctr2 == 1 && KEYPAD() == code[1]) ctr2 = 2;
if (ctr2 == 2 && KEYPAD() == code[2]) ctr2 = 3;
if (ctr2 == 3 && KEYPAD() == code[3]) ctr2 = 4;

// ENTER IDLE MODE ON KEYCODE
if (ctr2 == 4) {
PORTB.1 = 0;
PORTB.2 = 0;
ctr1 = 0;
ctr2 = 0;
ctr3 = 0;
mode = 0;
BEEP2();
}
}

 

// **********************************************************************************************
// **** MODE 3 : ALARM SIREN ACTIVATED
// **********************************************************************************************
while (mode == 3) {
   int i=0;
// TURN ON EXTERNAL ALARM
PORTB.0 = 1;

// POWER DOWN AND WAIT FOR CODE
//SLEEP();
 while (i<59){
 delay_ms(1000);
 i++;
}
PORTB.0 = 0;
SLEEP();
ctr1 = 0;
ctr2 = 0;
ctr3 = 0;
mode = 2;

}


// END OF MAIN LOOP
}
}